@@ -2,25 +2,32 @@
 #include "hw.h"
 #include "osutils.h"
 #include "version.h"
+#include "config.h"
 #include "options.h"
 #include "heuristics.h"
+#include "dump.h"
+#include "db.h"
+#include <cstring>
 #include <vector>
 #include <map>
 #include <sstream>
-#include <unistd.h>
+#include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
 #include <ctype.h>
+#include <limits.h>
 #include <unistd.h>
+#include <limits.h>
 #include <sys/utsname.h>
 
 using namespace hw;
 
-__ID("@(#) $Id: hw.cc 1818 2007-08-05 21:11:59Z lyonel $");
+__ID("@(#) $Id: hw.cc 2433 2012-01-10 22:01:30Z lyonel $");
 
 struct hwNode_i
 {
   hwClass deviceclass;
-  string id, vendor, product, version, serial, slot, handle, description,
+  string id, vendor, product, version, date, serial, slot, handle, description,
     businfo, physid, dev;
   bool enabled;
   bool claimed;
@@ -42,17 +49,24 @@ struct hwNode_i
 string hw::strip(const string & s)
 {
   string result = s;
+  size_t i = result.find('\0');
 
-  while ((result.length() > 0) && (result[0] <= ' '))
+  if(i != string::npos)
+    result = result.substr(0, i);
+
+  while ((result.length() > 0) && ((uint8_t)result[0] <= ' '))
     result.erase(0, 1);
-  while ((result.length() > 0) && (result[result.length() - 1] <= ' '))
-    result.erase(result.length() - 1, 1);
-  for (unsigned int i = 0; i < result.length(); i++)
-    if (result[0] <= ' ')
-  {
-    result.erase(i, 1);
-    i--;
-  }
+  while ((result.length() > 0) && ((uint8_t)result[result.length() - 1] <= ' '))
+    result.erase(result.length() - 1);
+
+  for (i = 0; i < result.length(); i++)
+    if ((uint8_t)result[i] < ' ')
+    {
+      result.erase(i, 1);
+      i--;
+    }
+
+  result = utf8_sanitize(result);
 
   return result;
 }
@@ -413,6 +427,21 @@ void hwNode::setVersion(const string & version)
     This->version = strip(version);
 }
 
+string hwNode::getDate() const
+{
+  if (This)
+    return This->date;
+  else
+    return "";
+}
+
+
+void hwNode::setDate(const string & s)
+{
+  if (This)
+    This->date = strip(s);
+}
+
 
 string hwNode::getSerial() const
 {
@@ -425,6 +454,9 @@ string hwNode::getSerial() const
 
 void hwNode::setSerial(const string & serial)
 {
+  if(serial == "00000000-0000-0000-0000-000000000000")
+    return;
+
   if (This)
     This->serial = strip(serial);
 }
@@ -876,6 +908,20 @@ string hwNode::getCapabilities() const
 }
 
 
+vector < string > hwNode::getCapabilitiesList() const
+{
+  vector < string > result;
+
+  if (!This)
+    return result;
+
+  for (unsigned int i = 0; i < This->features.size(); i++)
+    result.push_back(This->features[i]);
+
+  return result;
+}
+
+
 string hwNode::getCapabilityDescription(const string & feature) const
 {
   string featureid = cleanupId(feature);
@@ -977,19 +1023,22 @@ vector<string> hwNode::getLogicalNames() const
 
 void hwNode::setLogicalName(const string & name)
 {
+  string n = strip(name);
+
   if (This)
   {
     for (unsigned i = 0; i < This->logicalnames.size(); i++)
-      if (This->logicalnames[i] == strip(name) || This->logicalnames[i] == "/dev/" + strip(name))
+      if (This->logicalnames[i] == n || This->logicalnames[i] == "/dev/" + n)
         return;                                   // nothing to add, this logical name already exists
-    if (exists("/dev/" + strip(name)))
+    if ((name[0] != '/') && exists("/dev/" + n))
     {
-      This->logicalnames.push_back("/dev/" + strip(name));
+      This->logicalnames.push_back("/dev/" + n);
     }
     else
-      This->logicalnames.push_back(strip(name));
+      This->logicalnames.push_back(n);
 
-    This->dev = get_devid(strip(name));
+    if(This->dev == "")
+      This->dev = get_devid(n);
   }
 }
 
@@ -1139,8 +1188,11 @@ void hwNode::fixInconsistencies()
   if (getCapacity() < getSize())
     setCapacity(0);
 
-  if(getVendor() == "")
-    guessVendor(*this);
+  guessProduct(*this);
+  guessVendor(*this);
+
+  if(getLogicalName() != "")
+    claim();
 
   for (unsigned int i = 0; i < This->children.size(); i++)
     This->children[i].fixInconsistencies();
@@ -1275,6 +1327,300 @@ value hwNode::getHint(const string & id) const
   return This->hints[id];
 }
 
+vector < string > hwNode::getHints() const
+{
+  vector < string > result;
+
+  if (!This)
+    return result;
+
+  for (map < string, value >::iterator i = This->hints.begin();
+    i != This->hints.end(); i++)
+  result.push_back(i->first);
+
+  return result;
+}
+
+string hwNode::asJSON(unsigned level)
+{
+  vector < string > config;
+  vector < string > resources;
+  ostringstream out;
+
+  if(!This) return "";
+
+  config = getConfigKeys();
+  resources = getResources("\" value=\"");
+
+  if(visible(getClassName()))
+  {
+    out << "{" << endl;
+    out << spaces(2*level+2) << "\"id\" : \"" << getId() << "\"," << endl;
+    out << spaces(2*level+2) << "\"class\" : \"" << getClassName() << "\"";
+
+    if (disabled())
+      out << "," << endl << spaces(2*level+2) << "\"disabled\" : true";
+    if (claimed())
+      out << "," << endl << spaces(2*level+2) << "\"claimed\" : true";
+
+    if(getHandle() != "")
+      out << "," << endl << spaces(2*level+2) << "\"handle\" : \"" << getHandle() << "\"";
+
+    if (getDescription() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"description\" : \"";
+      out << escapeJSON(getDescription());
+      out << "\"";
+    }
+
+    if (getProduct() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"product\" : \"";
+      out << escapeJSON(getProduct());
+      out << "\"";
+    }
+
+    if (getVendor() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"vendor\" : \"";
+      out << escapeJSON(getVendor());
+      out << "\"";
+    }
+
+    if (getPhysId() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"physid\" : \"";
+      out << getPhysId();
+      out << "\"";
+    }
+
+    if (getBusInfo() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"businfo\" : \"";
+      out << escapeJSON(getBusInfo());
+      out << "\"";
+    }
+
+    if (getLogicalName() != "")
+    {
+      vector<string> logicalnames = getLogicalNames();
+
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"logicalname\" : ";
+      if(logicalnames.size() > 1)
+      {
+        out << "[";
+        for(unsigned int i = 0; i<logicalnames.size(); i++)
+        {
+          if(i) out << ", ";
+          out << "\"" << logicalnames[i] << "\"";
+        }
+        out << "]";
+      }
+      else
+        out << "\"" << escapeJSON(getLogicalName()) << "\"";
+    }
+
+    if (getDev() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"dev\" : \"";
+      out << escapeJSON(getDev());
+      out << "\"";
+    }
+
+    if (getVersion() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"version\" : \"";
+      out << escapeJSON(getVersion());
+      out << "\"";
+    }
+
+    if (getDate() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"date\" : \"";
+      out << escapeJSON(getDate());
+      out << "\"";
+    }
+
+    if (getSerial() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"serial\" : \"";
+      out << (::enabled("output:sanitize")?REMOVED:escapeJSON(getSerial()));
+      out << "\"";
+    }
+
+    if (getSlot() != "")
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"slot\" : \"";
+      out << escapeJSON(getSlot());
+      out << "\"";
+    }
+
+    if ((getSize() > 0) || (getCapacity() > 0))
+      switch (getClass())
+      {
+        case hw::memory:
+        case hw::address:
+        case hw::storage:
+        case hw::disk:
+        case hw::display:
+          out << "," << endl << spaces(2*level+2) << "\"units\" : \"bytes\"";
+          break;
+
+        case hw::processor:
+        case hw::bus:
+        case hw::system:
+          out << "," << endl << spaces(2*level+2) << "\"units\" : \"Hz\"";
+          break;
+
+        case hw::power:
+          out << "," << endl << spaces(2*level+2) << "\"units\" : \"mWh\"";
+          break;
+
+        case hw::network:
+          out << "," << endl << spaces(2*level+2) << "\"units\" : \"bit/s\"";
+          break;
+
+        default:
+          break;
+      }
+
+    if (getSize() > 0)
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"size\" : ";
+      out << getSize();
+    }
+
+    if (getCapacity() > 0)
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"capacity\" : ";
+      out << getCapacity();
+    }
+
+    if (getWidth() > 0)
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"width\" : ";
+      out << getWidth();
+    }
+
+    if (getClock() > 0)
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"clock\" : ";
+      out << getClock();
+    }
+
+    if (config.size() > 0)
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"configuration\" : {" << endl;
+      for (unsigned int j = 0; j < config.size(); j++)
+      {
+        if(j) out << "," << endl;
+        out << spaces(2*level+4);
+        out << "\"" << escapeJSON(config[j]) << "\" : \"" << escapeJSON(getConfig(config[j])) << "\"";
+      }
+      out << endl << spaces(2*level+2);
+      out << "}";
+    }
+    config.clear();
+
+    splitlines(getCapabilities(), config, ' ');
+    if (config.size() > 0)
+    {
+      out << "," << endl;
+      out << spaces(2*level+2);
+      out << "\"capabilities\" : {" << endl;
+      for (unsigned int j = 0; j < config.size(); j++)
+      {
+        if(j) out << "," << endl;
+        out << spaces(2*level+4);
+        out << "\"" << escapeJSON(config[j]) << "\" : ";
+        if (getCapabilityDescription(config[j]) == "")
+        {
+          out << "true";
+        }
+        else
+        {
+          out << "\"";
+          out << escapeJSON(getCapabilityDescription(config[j]));
+          out << "\"";
+        }
+      }
+      out << endl << spaces(2*level+2);
+      out << "}";
+    }
+    config.clear();
+
+    if (0 && resources.size() > 0)
+    {
+      out << spaces(2*level+1);
+      out << "<resources>" << endl;
+      for (unsigned int j = 0; j < resources.size(); j++)
+      {
+        out << spaces(2*level+2);
+        out << "<resource type=\"" << escapeJSON(resources[j]) << "\" />";
+        out << endl;
+      }
+      out << spaces(2*level+1);
+      out << "</resources>" << endl;
+    }
+    resources.clear();
+  }
+
+  
+  if(countChildren()>0)
+  {
+    if(visible(getClassName()))
+      out << "," << endl << spaces(2*level+2) << "\"children\" : [" << endl;
+
+    for (unsigned int i = 0; i < countChildren(); i++)
+    {
+      out << spaces(2*level+4) << getChild(i)->asJSON(visible(getClassName()) ? level + 2 : 1);
+      if(visible(getChild(i)->getClassName()) && (i < countChildren()-1)) out << "," << endl;
+    }
+
+    if(visible(getClassName()))
+      out << endl << spaces(2*level+2) << "]";
+  }
+
+  if(visible(getClassName()))
+  {
+    out << endl << spaces(2*level);
+    out << "}";
+  }
+
+  return out.str();
+}
 
 string hwNode::asXML(unsigned level)
 {
@@ -1292,22 +1638,26 @@ string hwNode::asXML(unsigned level)
     struct utsname un;
 
     out << "<?xml version=\"1.0\" standalone=\"yes\" ?>" << endl;
-    out << "<!-- generated by lshw-" << getpackageversion() << " -->" <<
+    out << _("<!-- generated by lshw-") << getpackageversion() << " -->" <<
   #if defined(__GNUC__) && defined(__VERSION__)
-      endl << "<!-- GCC " << __VERSION__ << " -->" <<
+      endl << "<!-- GCC " << escapecomment(__VERSION__) << " -->" <<
   #endif
       endl;
 
     if(uname(&un) == 0)
-      out << "<!-- " << un.sysname << " " << un.release << " " << un.version << " " << un.machine << " -->" << endl;
-  #if defined(__GLIBC__)
+      out << "<!-- " << escapecomment(un.sysname) << " " << escapecomment(un.release) << " " << escapecomment(un.version) << " " << escapecomment(un.machine) << " -->" << endl;
+  #if defined(__GLIBC__) && defined(_CS_GNU_LIBC_VERSION)
     char version[PATH_MAX];
 
       if(confstr(_CS_GNU_LIBC_VERSION, version, sizeof(version))>0)
-        out << "<!-- GNU libc " << __GLIBC__ << " (" << version << ") -->" << endl;
+        out << "<!-- GNU libc " << __GLIBC__ << " (" << escapecomment(version) << ") -->" << endl;
   #endif
     if (geteuid() != 0)
-      out << "<!-- WARNING: not running as root -->" << endl;
+      out << _("<!-- WARNING: not running as root -->") << endl;
+
+    if(::enabled("output:list"))
+      out << "<list>" << endl;
+
   }
 
   if(visible(getClassName()))
@@ -1400,6 +1750,15 @@ string hwNode::asXML(unsigned level)
       out << endl;
     }
 
+    if (getDate() != "")
+    {
+      out << spaces(2*level+1);
+      out << "<date>";
+      out << escape(getDate());
+      out << "</date>";
+      out << endl;
+    }
+
     if (getSerial() != "")
     {
       out << spaces(2*level+1);
@@ -1428,6 +1787,7 @@ string hwNode::asXML(unsigned level)
         case hw::address:
         case hw::storage:
         case hw::disk:
+        case hw::volume:
         case hw::display:
           out << " units=\"bytes\"";
           break;
@@ -1438,6 +1798,10 @@ string hwNode::asXML(unsigned level)
           out << " units=\"Hz\"";
           break;
 
+        case hw::network:
+          out << " units=\"bit/s\"";
+          break;
+
         case hw::power:
           out << " units=\"mWh\"";
           break;
@@ -1549,7 +1913,7 @@ string hwNode::asXML(unsigned level)
       for (unsigned int j = 0; j < resources.size(); j++)
       {
         out << spaces(2*level+2);
-        out << "<resource type=\"" << escape(resources[j]) << "\" />";
+        out << "<resource type=\"" << resources[j] << "\" />";
         out << endl;
       }
       out << spaces(2*level+1);
@@ -1569,9 +1933,48 @@ string hwNode::asXML(unsigned level)
     out << "</node>" << endl;
   }
 
+  if((level==0) && ::enabled("output:list"))
+    out << "</list>" << endl;
+
+
   return out.str();
 }
 
+string hwNode::asString()
+{
+  string summary = "";
+  if(!This)
+    return "";
+
+  if(getClass() != hw::memory)
+    summary = getProduct();        // memory devices tend to have obscure product names
+  if (summary == "")
+    summary = getDescription();
+
+  if((getClass() == hw::memory) || (getClass() == hw::disk) || (getClass() == hw::storage) || (getClass() == hw::volume))
+  {
+    unsigned long long size = 0;
+    if(getClass() != hw::memory)
+    {
+      if(getCapacity())
+        size = getCapacity();
+      else
+      {
+        if(getSize())
+          size = getSize();
+      }
+    }
+    else
+    {
+      if(getSize())
+        size = getSize();
+    }
+    if(size)
+      summary = (getClass()==hw::disk?(decimalkilos(size)+"B"):kilobytes(size)) +" "+ summary;
+  }
+
+  return summary;
+}
 
 struct hw::resource_i
 {
@@ -1580,6 +1983,7 @@ struct hw::resource_i
   unsigned int ui1;
   unsigned long ul1, ul2;
   unsigned long long ull1, ull2;
+  bool b;
 
   int refcount;
 };
@@ -1681,7 +2085,7 @@ unsigned long end)
 
 
 resource resource::mem(unsigned long long start,
-unsigned long long end)
+unsigned long long end, bool prefetchable)
 {
   resource r;
 
@@ -1691,6 +2095,7 @@ unsigned long long end)
   r.This->type = hw::mem;
   r.This->ull1 = start;
   r.This->ull2 = end;
+  r.This->b = prefetchable;
 
   return r;
 }
@@ -1737,30 +2142,34 @@ string resource::asString(const string & separator) const
   switch (This->type)
   {
     case hw::none:
-      result = "(none)";
+      result = _("(none)");
       break;
     case hw::dma:
-      result = "dma" + separator;
+      result = _("dma") + separator;
       snprintf(buffer, sizeof(buffer), "%d", This->ui1);
       break;
     case hw::irq:
-      result = "irq" + separator;
+      result = _("irq") + separator;
       snprintf(buffer, sizeof(buffer), "%d", This->ui1);
       break;
     case hw::iomem:
-      result = "iomemory" + separator;
+      result = _("iomemory") + separator;
       snprintf(buffer, sizeof(buffer), "%llx-%llx", This->ull1, This->ull2);
       break;
     case hw::mem:
-      result = "memory" + separator;
+      result = _("memory") + separator;
       snprintf(buffer, sizeof(buffer), "%llx-%llx", This->ull1, This->ull2);
+      if(This->b) strcat(buffer, _("(prefetchable)"));
       break;
     case hw::ioport:
-      result = "ioport" + separator;
-      snprintf(buffer, sizeof(buffer), "%lx-%lx", This->ul1, This->ul2);
+      result = _("ioport") + separator;
+      if(This->ul1 == This->ul2)
+        snprintf(buffer, sizeof(buffer), "%lx", This->ul1);
+      else
+        snprintf(buffer, sizeof(buffer), _("%lx(size=%ld)"), This->ul1, This->ul2 - This->ul1 + 1);
       break;
     default:
-      result = "(unknown)";
+      result = _("(unknown)");
   }
 
   return result + string(buffer);
@@ -1786,14 +2195,17 @@ const
       return This->ui1 == r.This->ui1;
       break;
 
-    case hw::iomem:case hw::mem:return (This->ull1 >= r.This->ull1)
-      && (This->ull2 <= r.This->ull2) || (r.This->ull1 >= This->ull1)
-      && (r.This->ull2 <= This->ull2);
+    case hw::iomem:
+    case hw::mem:
+      return ((This->ull1 >= r.This->ull1)
+      && (This->ull2 <= r.This->ull2)) || ((r.This->ull1 >= This->ull1)
+      && (r.This->ull2 <= This->ull2));
     break;
 
-    case hw::ioport:return (This->ul1 >= r.This->ul1)
-      && (This->ul1 <= r.This->ul2) || (r.This->ul1 >= This->ul1)
-      && (r.This->ul1 <= This->ul2);
+    case hw::ioport:
+      return ((This->ul1 >= r.This->ul1)
+      && (This->ul1 <= r.This->ul2)) || ((r.This->ul1 >= This->ul1)
+      && (r.This->ul1 <= This->ul2));
     break;
 
     default:return false;
@@ -1923,13 +2335,13 @@ string value::asString() const
   switch(This->type)
   {
     case hw::integer:
-      return "0";
+      return "0x"+tohex(This->ll);
     case hw::text:
       return This->s;
     case hw::boolean:
-      return This->b?"true":"false";
+      return This->b?_("true"):_("false");
     case hw::nil:
-      return "(nil)";
+      return _("(nil)");
   };
 
   return "";
@@ -1943,3 +2355,24 @@ bool value::defined() const
 
   return true;
 }
+
+bool hwNode::dump(const string & filename, bool recurse)
+{
+#ifdef SQLITE
+  try {
+    sqlite::database db(filename);
+
+    db.execute("BEGIN TRANSACTION");
+    ::dump(*this, db, "", recurse);
+    db.execute("COMMIT");
+  }
+  catch(exception & e)
+  {
+    return false;
+  }
+#else
+  return false;
+#endif
+
+  return true;
+}
